/*
 nRF24 RC Car RX
 Version: 0.1
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 2/11/2013
 Last Modified: 10/11/2013
 
 Receives the commands from the TX side, controls a servo and motor.
 
 */

// ATtiny24/44/84 Pin map
//
//                              +-\/-+
//                        VCC  1|o   |14  GND
//                        PB0  2|    |13  PA0 433MHz RF In
//                        PB1  3|    |12  PA1 
//                  Reset PB3  4|    |11  PA2 
//  Motor forward control PB2  5|    |10  PA3 
//  Motor reverse control PA7  6|    |9   PA4 Back lights
//       Servo Left/Right PA6  7|    |8   PA5 Front lights
//                              +----+

#define F_CPU 1000000 // 1 MHz

#define FOWARD_CONTROL PA1
#define REVERSE_CONTROL PA2
#define RF_IN PA0
#define FRONT_LIGHTS PA5
#define BACK_LIGHTS PA4

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include <string.h>
#include "setup.c"

uint8_t data_in[4]; // 0: Preamble, 1: Left/Right, 2: Forward/Reverse, 3: CRC
uint8_t wifiTimeout = 0; // A timeout for stopping the motor

int main(void) {
	setup();
	
	while(1) {
		// Receive data
		if (rf433_receive_data()) {
			// Left/Right control
			if (data_in[LEFT_RIGHT_CTRL] > 150) { // Right
				uint8_t leftrightReading = data_in[LEFT_RIGHT_CTRL] - 150;
				OCR1A = SERVO_CENTER + leftrightReading + 60;
			}
			else if (data_in[LEFT_RIGHT_CTRL] < 100) { // Left
				uint8_t leftrightReading = 100 - data_in[LEFT_RIGHT_CTRL];
				OCR1A = SERVO_CENTER - leftrightReading - 70;
				if (data_in[LEFT_RIGHT_CTRL] <= 10) { // Max out
					OCR1A = SERVO_CENTER - leftrightReading - 100;
				}
			}
			else {
				OCR1A = SERVO_CENTER;
			}
			
			// Motor control
			int forwardreverseReading = data_in[FORWARD_REVERSE_CTRL];
			if (forwardreverseReading > 150) { // Forward
				// Fine tune the motor control
				uint8_t calcOcr = 0;
				for (int x = 0; x < (forwardreverseReading-150); x++) {
					calcOcr += 2;
				}
				if (calcOcr > 170) { 
					calcOcr = 255;
				}
				
				OCR0B = 0; // Turn off reverse mosfets
				OCR0A = calcOcr;
			}
			else if (forwardreverseReading < 100) { // Reverse
				// Fine tune the motor control
				uint8_t calcOcr = 0;
				for (int x = 0; x < (100 - forwardreverseReading); x++) {
					calcOcr += 2;
				}
				if (calcOcr > 170) { 
					calcOcr = 255;
				}
				
				OCR0A = 0; // Turn off forward mosfets
				OCR0B = calcOcr;
			}
			else {
				OCR0A = 0;
				OCR0B = 0;
			}
			
			// Lights
			if (data_in[LIGHTS_AND_CRC_CHECK] & 0x80) {
				PORTA |= (1<<FRONT_LIGHTS);
			}
			else {
				PORTA &= ~(1<<FRONT_LIGHTS);
			}
			if (data_in[LIGHTS_AND_CRC_CHECK] & 0x40) {
				PORTA |= (1<<BACK_LIGHTS);
			}
			else {
				PORTA &= ~(1<<BACK_LIGHTS);
			}
			
			wifiTimeout = 0;
		}
		else {
			wifiTimeout++;
		}
		
		// If we are no longer receiving data, turn everything off
		if (wifiTimeout >= 50) {
			OCR0A = 0;
			OCR0B = 0;
			OCR1A = SERVO_CENTER;
			wifiTimeout = 50;
		}
	}
}
