/*
 nRF24 RC Car RX
 Version: 0.1
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 2/11/2013
 Last Modified: 10/11/2013

 */
 
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef boolean
	typedef uint8_t boolean;
#endif
#ifndef bool
	typedef uint8_t bool;
#endif 
#ifndef byte
	typedef uint8_t byte;
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

#define SERVO_RIGHT 350
#define SERVO_CENTER 625
#define SERVO_LEFT 950

#define PREAMBLE 0
#define LEFT_RIGHT_CTRL 1
#define FORWARD_REVERSE_CTRL 2
#define LIGHTS_AND_CRC_CHECK 3

extern uint8_t data_in[4];
uint8_t comparePreamble = 173; // Preamble

// Receive data
uint8_t rf433_receive_data(void) {
	// Clear the data_in
	data_in[PREAMBLE] = 0;
	data_in[LEFT_RIGHT_CTRL] = 0;
	data_in[FORWARD_REVERSE_CTRL] = 0;
	data_in[LIGHTS_AND_CRC_CHECK] = 0;
	
	// Wait for blanking of 500uS
	uint8_t blanking = 0;
	while (blanking <= 50) {
		if (!(PINA & (1<<RF_IN))) {
			blanking++;
		}
		else {
			blanking = 0;
		}
		_delay_us(5);
	}
	
	// Wait for first high transition
	while (!(PINA & (1<<RF_IN)));
	
	// Read data
	for (uint8_t x = 0; x < 4; x++) {
		int8_t bitCounter = 8;
		uint8_t checksum = 0;
		while (bitCounter >= 0) {
			// Wait for high transition
			uint8_t timeCounter = 0;
			while (!(PINA & (1<<RF_IN))) {
				_delay_us(1);
				timeCounter++;
				if (timeCounter >= 100) {
					return 0;
				}
			}
			
			// Start timer to check how long it's high for
			timeCounter = 0;
			for (timeCounter = 0; timeCounter <= 66; timeCounter++) {
				_delay_us(1);
				if (!(PINA & (1<<RF_IN))) { // Break when it goes low
					break;
				}
			}
			if (timeCounter >= 66) { // High for too long, exit
				return 0;
			}
			
			// Process counter
			if (bitCounter >= 1) {
				if (timeCounter >= 10 && timeCounter <= 35) { // 0 bit
					checksum ^= 0;
				}
				else if (timeCounter >= 36 && timeCounter <= 65) { // 1 bit
					data_in[x] |= (1<<(bitCounter-1));
					checksum ^= 1;
				}
			}
			else { // Check the checksum
				if (timeCounter >= 10 && timeCounter <= 35 && checksum != 0) {
					return 0;
				}
				else if (timeCounter >= 36 && timeCounter <= 65 && checksum != 1) {
					return 0;
				}
			}
			bitCounter--;
		}
	}
	
	// Check Preamble matches
	if (data_in[PREAMBLE] != comparePreamble) {
		return 0;
	}
	
	// Check 6 bit CRC matches
	uint8_t crc6Bit = 0;
	for (uint8_t x = 0; x < 3; x++) {
		crc6Bit += data_in[x];
	}
	
	// Only care about low 6 bits
	crc6Bit = crc6Bit & 0x3F;
	uint8_t data_in_crc6Bit = data_in[LIGHTS_AND_CRC_CHECK] & 0x3F;
	if (crc6Bit != data_in_crc6Bit) {
		return 0;
	}

	return 1;
}

// Setup the ATtiny
void setup(void) {
	// Set up servo on Timer1 16bit
	DDRA |= (1<<PA6);
	TCNT1 = 0;
	ICR1 = 10000; // Sets frequency to 50Hz / 20ms
	OCR1A = SERVO_CENTER; // Sets 1ms (servo centre)
	TCCR1A |= (1<<COM1A1);
	TCCR1B = (1<<WGM13) | (1<<CS10); // PWM, Phase & Freq. Correct, 1 prescaler
	
	// Set up motor on Timer0 8 bit
	DDRB |= (1<<PB2); // Motor forward control
	DDRA |= (1<<PA7); // Motor reverse control
	TCNT0 = 0;
	OCR0A = 0; // Sets motor forward duty cycle to 0%
	OCR0B = 0; // Sets motor reverse duty cycle to 0%
	TCCR0A |= (1<<COM1A1) | (1<<COM1B1) | (1<<WGM00); // PWM, Phase Correct and OCRA/B outputs
	TCCR0B = (1<<CS01); // 8 prescaler
	
	// Lights
	DDRA |= (1<<FRONT_LIGHTS) | (1<<BACK_LIGHTS);
	
	sei(); // Turn on interrupts
}
