/*
 433MHz RC Car TX
 Version: 0.1
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 27/12/2014
 Last Modified: 28/12/2014
 
 Transmit the ADC values of the 2 pots to the receiver.
 
 */
// ATtiny24/44/84 Pin map
//
//                              +-\/-+
//                        VCC  1|o   |14  GND
//                        PB0  2|    |13  PA0 ADC0 Forward/Reverse
//                        PB1  3|    |12  PA1 ADC1 Left/Right
//                  Reset PB3  4|    |11  PA2 Backlights
//                        PB2  5|    |10  PA3 Headlights
//                        PA7  6|    |9   PA4
//                        PA6  7|    |8   PA5 433MHz RF Out
//                              +----+

#define F_CPU 1000000 // 1 MHz

#define forwardreverseADCPin 0
#define leftrightADCPin 1
#define HEAD_LIGHTS PA3
#define BACK_LIGHTS PA2

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include <string.h>
#include "setup.c"

uint8_t frontLightsHeld = 0;
uint8_t backLightsHeld = 0;

int main(void) {
	setup();
	
	while(1) {
		// Read the voltage of the pots
		data_out[LEFT_RIGHT_CTRL] = analog_read_ADCH(leftrightADCPin);
		data_out[FORWARD_REVERSE_CTRL] = analog_read_ADCH(forwardreverseADCPin);
		
		// Head lights
		data_out[LIGHTS_AND_CRC_CHECK] = 0;
		if (PINA & (1<<HEAD_LIGHTS)) {
			data_out[LIGHTS_AND_CRC_CHECK] |= 0x80;
			
			frontLightsHeld++;
			if (frontLightsHeld >= 100) { // Turn lights off from being on constantly if button pressed
				frontLightsHeld = 0;
			}
			else if (frontLightsHeld >= 50) {
				frontLightsHeld = 50;
			}
		}
		else if (frontLightsHeld < 50) {
			frontLightsHeld = 0;
		}
		if (frontLightsHeld >= 50) { // If lights held down for a while, keep them on
			data_out[LIGHTS_AND_CRC_CHECK] |= 0x80;
			frontLightsHeld++;
			if (frontLightsHeld >= 100) {
				frontLightsHeld = 100;
			}
		}
		
		// Back lights
		if (PINA & (1<<BACK_LIGHTS)) {
			data_out[LIGHTS_AND_CRC_CHECK] |= 0x40;
			
			backLightsHeld++;
			if (backLightsHeld >= 100) {
				backLightsHeld = 0;
			}
			else if (backLightsHeld >= 50) {
				backLightsHeld = 50;
			}
		}
		else if (backLightsHeld < 50) {
			backLightsHeld = 0;
		}
		if (backLightsHeld >= 50) {
			data_out[LIGHTS_AND_CRC_CHECK] |= 0x40;
			backLightsHeld++;
			if (backLightsHeld >= 100) {
				backLightsHeld = 100;
			}
		}
		
		// Transmit
		rf433_transmit_data();
	}
}

ISR(ADC_vect) { }
