/*
 433MHz RC Car TX
 Version: 0.1
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 27/12/2014
 Last Modified: 28/12/2014
 
 */
 
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef boolean
	typedef uint8_t boolean;
#endif
#ifndef bool
	typedef uint8_t bool;
#endif 
#ifndef byte
	typedef uint8_t byte;
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

#define PREAMBLE 0
#define LEFT_RIGHT_CTRL 1
#define FORWARD_REVERSE_CTRL 2
#define LIGHTS_AND_CRC_CHECK 3

#define RF_OUT PA5
#define rf_433_high	PORTA |=  (1<<RF_OUT);
#define rf_433_low	PORTA &= ~(1<<RF_OUT);
#define RF_BIT1_DELAY 400
#define RF_BIT0_DELAY 200
#define RF_BLANK_DELAY 200

uint8_t data_out[4]; // First byte: Left/Right, Second byte: Forward/Reverse
uint8_t preamble = 173; // Preamble

// Send data
void rf433_transmit_data(void) {
	// Blanking period
	rf_433_low;
	_delay_us(500);
	
	// 6 bit CRC calculation
	uint8_t crc6Bit = 0;
	for (uint8_t x = 0; x < 3; x++) {
		crc6Bit += data_out[x];
	}
	
	// Only care about low 6 bits
	crc6Bit = crc6Bit & 0x3F;
	data_out[LIGHTS_AND_CRC_CHECK] |= crc6Bit;
	
	// Transmit data out
	for (uint8_t x = 0; x < 4; x++) {
		uint8_t checksum = 0;
		for (int8_t i = 7; i >= 0; i--) {
			if (data_out[x] & (1<<i)) { // High bit
				rf_433_high;
				_delay_us(RF_BIT1_DELAY);
				rf_433_low;
				_delay_us(RF_BLANK_DELAY);
				checksum ^= 1;
				
			}
			else { // Low bit
				rf_433_high;
				_delay_us(RF_BIT0_DELAY);
				rf_433_low;
				_delay_us(RF_BLANK_DELAY);
				checksum ^= 0;
			}
		}
		
		// Transmit checksum
		if (checksum == 1) {
			rf_433_high;
			_delay_us(RF_BIT1_DELAY);
			rf_433_low;
			_delay_us(RF_BLANK_DELAY);
			
		} else {
			rf_433_high;
			_delay_us(RF_BIT0_DELAY);
			rf_433_low;
			_delay_us(RF_BLANK_DELAY);
		}
	}
	
	rf_433_low;
}

// Sleep
void system_sleep(void) {
	set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Set sleep mode
	sleep_mode(); // System sleeps here
}

// ADC Read
int analog_read_ADCH(uint8_t pin) {	
	// Set the analog pin and voltage reference as VCC
	ADMUX = (pin & 0x3f);
	
	set_sleep_mode(SLEEP_MODE_ADC); // Set sleep mode ADC
	sleep_mode(); // System sleeps here
	
	return ADCH;
}

// Setup the ATtiny
void setup(void) {
	// ADC Left Adjust Result for 8 bit resolution
	sbi(ADCSRB, ADLAR);
	
	// Turn on ADC interrupt
	sbi(ADCSRA, ADIE);
	
	// Turn on ADC
	sbi(ADCSRA, ADEN);
	
	sei(); // Turn on interrupts
	
	// Set as RF output
	DDRA |= (1<<RF_OUT);
	
	// Set preamble
	data_out[PREAMBLE] = preamble;
}
